/*

xlib_shm viewer for zm 1.22.3 version 0.6.3 shows images directly from shm memory
 
Copyright (C) maciekc 
maciekcz@(NO/SPAM)epf.pl
 
This program is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. 
 
This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
GNU General Public License for more details. 
 
You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software 
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
------------------------------------------------------------------------------ 


CHANGELOG:
0.6.3	-	support for zm 1.22.3
0.6.2	-	add gray pallete support
		support to close from wm event
		added info about error genereated by shmget
0.6.1	-	fixed problem with seg fault
0.6	-	fixed problem with setting higher mode than at start
	-	fallback to ximage when rgb32 xv is not available
0.5.1	-	fixed high cpu usage
0.5	-	added modes - single, quad, 9monitors and 16monitors
		changing monitor and modes with keyboard
		keyboard support
		code reformatted and a little bit cleaned
0.4.1	-	quick fix for zm 1.22.1
0.4	-	xvideo support - auto image rescaling for full screen display
0.3	-	multi monitor support up to 32
0.2	-	reading from zm.conf and database - you don't need to specify 
		anything expect monitor_id in command line
0.1.2 	-	fixed memory leak (ximage not beeing destroyed)
0.1.1 	- 	added -b option ring buffer, fixed some memory allocation problems, copyright & license info
0.1 	-	first version

Sorry for the code. It is written very badly, but works:)
It had to be rewritten if I have time.

*/
#define VERSION "0.6.3"
#define ZM_CONFIG "/etc/zm.conf"


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdbool.h>
#include <math.h>
#include <sys/shm.h>
#include <mysql/mysql.h>
#include <string.h>
#include <errno.h>

//xv stuff
#include <X11/extensions/Xvlib.h>
#define RGB24 0x3


#define OPTIONS "x:y:k:m:b:d:f:h:w:c:s i"
//zm shm structures

#define key1 10
#define key2 11
#define key3 12
#define key4 13
#define key5 14
#define keyq 24
#define keyw 25
#define keye 26
#define keyr 27
#define keyt 28
#define keya 38
#define keys 39
#define keyd 40
#define keyf 41
#define keyg 42
#define keyz 52
#define keyx 53
#define keyc 54
#define keyv 55
#define keyb 56

#define RGB 0
#define GRAY 1

typedef enum 
{ 
    IDLE,
    PREALARM,
    ALARM,
    ALERT,
    TAPE
} State;

typedef struct
{
    int size;
    bool valid;
    bool active;
    bool signal;
    State state;
    int last_write_index;
    int last_read_index;
    time_t last_image_time;
    int last_event;
    int action;
    int brightness;
    int hue;
    int colour;
    int contrast;
    int alarm_x;
    int alarm_y;
    char control_state[256];
} SharedData; 


typedef enum { TRIGGER_CANCEL, TRIGGER_ON, TRIGGER_OFF } TriggerState;

typedef struct
{
    int size;
    TriggerState trigger_state;
    int trigger_score;
    char trigger_cause[32];
    char trigger_text[256];
    char trigger_showtext[256];
} TriggerData;

typedef struct
{
    int *image_buffer_count;
    int *width;
    int *height;
    int mon_id;
    int pallete;
    //for future use
    // int *fps[32];
    XvImage *image;
    SharedData *shared_data;
    unsigned char *shared_images;
    //int lastread;
} MONITOR;

XImage *ximage;


typedef struct
{
    int mon_num;
    int lastread;
} BOX;
MONITOR *monitor[32];

char *server;
char *user;
char *password;
char *database;

int int_temp;
Atom wm_delete_window;
//main


static int GetPortId(Display *dpy)
{
    int i, j, k, numAdapt, numImages, portNum;
    XvImageFormatValues *formats;
    XvAdaptorInfo *info;
    
    portNum = -1;
    
    if(Success != XvQueryAdaptors(dpy,DefaultRootWindow(dpy),&numAdapt,&info)) return -1;
    
    fprintf(stderr, "Found %i Xv adaptors\n", numAdapt);
    
    for(i = 0; i < numAdapt; i++)
    {
	if(info[i].type & XvImageMask)
	{
	    /* Adaptor has XvImage support */
	    formats = XvListImageFormats(dpy, info[i].base_id, &numImages);
	    
	    for(j = 0; j < numImages; j++) 
	    {
		if(formats[j].id == RGB24)
		{
		    /* It supports our format */
		    for(k = 0; k < info[i].num_ports; k++)
		    {
			/* try to grab a port */
			if(Success == XvGrabPort(dpy, info[i].base_id + k, CurrentTime))
			{
			    portNum = info[i].base_id + k;
			    break;
			}
		    }
		}
		if(portNum != -1) break;
	    }
	    XFree(formats);
	}
    if(portNum != -1) break;
    }
    
    XvFreeAdaptorInfo(info);
    return portNum;
}

// this stuff is almost without change from zm_config.cpp
void loadzmconfig()
{
    FILE *cfg;
    char line[512];
    char *val;
    int r;
    if ( (cfg = fopen( ZM_CONFIG, "r")) == NULL )
    {
	fprintf(stderr,"Can't open %s: %m",ZM_CONFIG);
	exit(1);
    }
    while ( fgets( line, sizeof(line), cfg ) != NULL )
    {
	char *line_ptr = line;
	// Trim off any cr/lf line endings
	int chomp_len = strcspn( line_ptr, "\r\n" );
	line_ptr[chomp_len] = '\0';
	
	// Remove leading white space
	int white_len = strspn( line_ptr, " \t" );
	line_ptr += white_len;
	
	// Check for comment or empty line
	if ( *line_ptr == '\0' || *line_ptr == '#' )
	continue;
	// Remove trailing white space
	char *temp_ptr = line_ptr+strlen(line_ptr)-1;
	while ( *temp_ptr == ' ' || *temp_ptr == '\t' )
	{
	    *temp_ptr-- = '\0';
	    temp_ptr--;
	}
	
	// Now look for the '=' in the middle of the line
	temp_ptr = strchr( line_ptr, '=' );
	if ( !temp_ptr )
	{
	    fprintf(stderr,"Invalid data in %s: '%s'\n", ZM_CONFIG, line );
	    continue;
	}
	
	// Assign the name and value parts
	char *name_ptr = line_ptr;
	char *val_ptr = temp_ptr+1;
	
	// Trim trailing space from the name part
	do
	{
	    *temp_ptr = '\0';
	    temp_ptr--;
	}
	while ( *temp_ptr == ' ' || *temp_ptr == '\t' );
	
	// Remove leading white space from the value part
	white_len = strspn( val_ptr, " \t" );
	val_ptr += white_len;
	
	val = (char *)malloc( strlen(val_ptr)+1 );
	strncpy( val, val_ptr, strlen(val_ptr)+1 );
	
	if ( strcasecmp( name_ptr, "ZM_DB_HOST" ) == 0 ) server=val;
	else if ( strcasecmp( name_ptr, "ZM_DB_NAME" ) == 0 ) database=val;
	else if ( strcasecmp( name_ptr, "ZM_DB_USER" ) == 0 ) user = val;
	else if ( strcasecmp( name_ptr, "ZM_DB_PASS" ) == 0 ) password=val;
	else
	{
	    // We ignore this now as there may be more parameters than the
	    // c/c++ binaries are bothered about
	    // Warning(( "Invalid parameter '%s' in %s", name_ptr, ZM_CONFIG )
	}
    }
    fclose(cfg);
}

int getmonitordata(int *w, int *h, int *b, int *f, int id)
{
    MYSQL *conn;
    MYSQL_RES *res;
    MYSQL_ROW row;
    
    conn = mysql_init(NULL);
    
    if (!mysql_real_connect(conn, server, user, password, database, 0, NULL, 0))
    {
	fprintf(stderr, "%s\n", mysql_error(conn));
	exit(EXIT_FAILURE);
    }
    
    char *query=malloc(300);
    sprintf(query,"SELECT `Width`, `Height`, `ImageBufferCount`, `MaxFPS` FROM `Monitors` WHERE `Id`=%d",id);
    
    if (mysql_query(conn, query))
    {
	fprintf(stderr, "%s\n", mysql_error(conn));
	exit(EXIT_FAILURE);
    }
    
    res = mysql_use_result(conn);
    
    if(row = mysql_fetch_row(res))
    {
	*w=strtol(row[0],NULL,0);
	*h=strtol(row[1],NULL,0);
	*b=strtol(row[2],NULL,0);
	if (f!=NULL) *f=strtol(row[3],NULL,0);
    }
    else
    {
	fprintf(stdout,"ERROR: specified monitor does not exists in database\n");
	exit(EXIT_FAILURE);
    }

    free(query);
    mysql_free_result(res);
    mysql_close(conn);

}



int main (int argc, char *argv[])
{
    int x,y;
    int imgx=0,imgy=0;
    int w=0;
    int h=0;
    int max_width=0;
    int max_height=0;
    long long shm_key=0x7a6d2000;
    int cam;
    int cam_num=0;
    int cam_in_row=2;
    //int cam_numf=0;
    long delay=(pow(10,6)/25)/4;
    int arg;
    bool xvdis=0;
    
    
    while((arg=getopt(argc, argv, OPTIONS)) != -1) switch(arg)
    {
	case 'd':
	    delay=strtol(optarg,NULL,0);
	    break;
	case 'w': 
	    w=strtol(optarg, NULL,0);
	    break;
	case 'h':
	    h=strtol(optarg, NULL,0);
	    break;
	case 'x':
	    imgx=strtol(optarg, NULL,0);
	    break;
	case 'y':
	    imgy=strtol(optarg, NULL,0);
	    break;
	case 'k':
	    shm_key=strtol(optarg, NULL,0);
	    break;
	case 'm':
	    monitor[cam_num]=malloc(sizeof(MONITOR));
	    monitor[cam_num]->mon_id=strtol(optarg, NULL,0);
	    monitor[cam_num]->pallete=RGB;
	    cam_num++;
	    break;
	case 'b':
	    monitor[cam_num]=malloc(sizeof(MONITOR));
	    monitor[cam_num]->mon_id=strtol(optarg, NULL,0);
	    monitor[cam_num]->pallete=GRAY;
	    cam_num++;
	    break;
	case 'f':
	    //for future use
	    fprintf(stdout,"-f - fps for monitor - in the future(NOW NOT WORKING)");
	    //monitor[cam_numf].fps=malloc(sizeof(int));
	    //*monitor[cam_numf].fps=strtol(optarg, NULL,0);
	    //cam_numf++;
	    break;
	case 'c':
	    cam_in_row=strtol(optarg, NULL,0);
	    break;
	case 'i':
	    fprintf(stdout,"xlib_shm viewer for zm 1.22.3 version ");
	    fprintf(stdout,VERSION);
	    fprintf(stdout,"\nshows images directly from shm memory\n");
	    fprintf(stdout,"-x - x position of first monitor - default 0\n");
	    fprintf(stdout,"-y - y position of first monitor - default 0\n");
	    fprintf(stdout,"-w - width of a window - default x resolution\n");
	    fprintf(stdout,"-h - height of a window - default y resolution\n");
	    fprintf(stdout,"-k - shared memory key - shm_key - default 0x7a6d2000\n");
	    fprintf(stdout,"-m - monitor id(max 32) - RGB24\n");
	    fprintf(stdout,"-b - monitor id(max 32) - GRAY\n");
	    fprintf(stdout,"-d - delay between getting images in us - default 10000\n");
	    fprintf(stdout,"-c - how many cameras in row and column - 1 single, 2 - quad, 3 - 9monitors, 4 - 16monitors\n");
	    fprintf(stdout,"-i - this info you are now reading, read README for more\n");
	    return 0;
	    break;
	case 's':
	    fprintf(stdout,"Disabling xv\n");
	    xvdis=1;
	    break;
    }
    
    fprintf(stdout,"xlib_shm viewer for zm 1.22.3 version ");
    fprintf(stdout,VERSION);
    fprintf(stdout,"\n");
    
    if(monitor[0]==NULL)
    {
	fprintf(stderr,"No monitor id specified\n-i for info\n");
	return EXIT_FAILURE;
    }
    
    loadzmconfig();
    
    //read all specified monitors from database
    if (cam_num>32)
    {
	fprintf(stderr,"ERROR: Too many monitors specified\n");
    }

    if(cam_in_row>4||cam_in_row<1)
    {
	fprintf(stdout,"bad value for cam_in_row");
	return EXIT_FAILURE;
    }
    
    for(cam=0;cam<cam_num;cam++)
    {
	monitor[cam]->width=malloc(sizeof(int));
	monitor[cam]->height=malloc(sizeof(int));
	monitor[cam]->image_buffer_count=malloc(sizeof(int));
	//for future use
	//monitor[cam]->fps=malloc(sizeof(int));
	
	//if (*fps[cam]==0)
	//{
	// getmonitordata(width[cam],height[cam],image_buffer_count[cam],fps[cam],mon_id[cam]);
	//}
	//else
	//{
	getmonitordata(monitor[cam]->width,monitor[cam]->height,monitor[cam]->image_buffer_count,NULL,monitor[cam]->mon_id);
	//}
    
	//look for the biggest camera
	if(*monitor[cam]->height>max_height) max_height=*monitor[cam]->height;
	if(*monitor[cam]->width>max_width) max_width=*monitor[cam]->width;
    }
    
    
    free(server);
    free(user);
    free(password);
    free(database);
    
    fprintf(stdout,"imgx=%d imgy=%d\ndelay=%ld\nshmkey=%llx\n",imgx,imgy,delay,shm_key);
    fprintf(stdout,"monitors:");
    cam=0;
    while(cam<cam_num)
    {
	fprintf(stdout,"%d ",monitor[cam]->mon_id);
	cam++;
    }
    fprintf(stdout,"\n");
    Display *dis;
    int screen_number;
    Window win;
    GC gc;
    Window parent;
    XEvent event;
    bool xv=1;
    
    dis=XOpenDisplay(NULL);
    if(dis==NULL)
    {
	fprintf(stderr,"ERROR: Unable to open display\n");
	return EXIT_FAILURE;
    }
    
    screen_number = DefaultScreen (dis);
    
    parent = RootWindow (dis,screen_number);
    if (w==0) w=XDisplayWidth(dis,screen_number);
    if (h==0) h=XDisplayHeight(dis, screen_number);
    win = XCreateSimpleWindow (dis,parent,0,0,w,h,2,0,0);
    
    if(win==None)
    {
	fprintf(stderr,"ERROR: Unable to create window\n");
	return EXIT_FAILURE;
    }
    
    wm_delete_window = XInternAtom(dis,"WM_DELETE_WINDOW",0);
    XSetWMProtocols(dis,win,&wm_delete_window,1);
    
    int portNum = -1;
    
    XMapWindow (dis, win);
    XMoveWindow(dis,win,0,0);
    gc = XCreateGC (dis, win, 0, NULL);
    //XSetForeground(dis,gc,WhitePixel(dis,screen_number));
    if(gc==NULL)
    {
	fprintf(stderr,"ERROR: Unable to create gc\n");
	return EXIT_FAILURE;
    }
    
    
    void *shm_ptr;
    unsigned char *rgba;
    rgba=malloc(max_width*max_height*4);

    if(xvdis!=1)
    {
	portNum = GetPortId(dis);
     
	if(portNum == -1)
	{
	    fprintf(stdout, "WARNING: Couldn't find free Xv adaptor with RGB XvImage support\n");
	    //XCloseDisplay(dis);
	    fprintf(stdout, "Falling back to ximage - slow and ugly rescalling\n");
	    xv=0;
	    //return EXIT_FAILURE;
	}
    }
    if(xvdis) xv=0;

    for(cam=0;cam<cam_num;cam++)
    {
	if(xv)
	{
    	    monitor[cam]->image = XvCreateImage(dis, portNum, RGB24, rgba, *monitor[cam]->width, *monitor[cam]->height);
	    if(monitor[cam]->image==NULL)
	    {
		fprintf(stdout,"WARNING: Unable to create xvimage\n");
		xv=0;
		fprintf(stdout,"Falling back to ximage - slow and ugly rescalling\n");
	    }
	}
	
	
	int shared_data_size = sizeof(SharedData) +
	sizeof(TriggerData) +
	((*monitor[cam]->image_buffer_count)*(sizeof(struct timeval)));
	if(monitor[cam]->pallete==GRAY) shared_data_size += ((*monitor[cam]->image_buffer_count)*(*monitor[cam]->width)*(*monitor[cam]->height)*1);
	if(monitor[cam]->pallete==RGB) shared_data_size += ((*monitor[cam]->image_buffer_count)*(*monitor[cam]->width)*(*monitor[cam]->height)*3);
	fprintf(stdout,"shared_data_size=%d\n",shared_data_size);
	int shmid;
	if((shmid=shmget((shm_key&0xffffff00)|monitor[cam]->mon_id,shared_data_size,SHM_R))==-1)
	{
	    fprintf(stderr,"ERROR: Failed to shmget\n");
	    switch(errno)
	    {
		case EINVAL: fprintf(stderr,"EINVAL - size < SHMMIN or size > SHMMAX\n"); break;
		case EEXIST: fprintf(stderr,"EEXIST - already exist\n"); break;
		case EIDRM: fprintf(stderr,"EIDRM - removed or set to be removed\n"); break;
		case ENOSPC: fprintf(stderr,"ENOSPC - shmmni or shmall limit reached\n"); break;
		case ENOENT: fprintf(stderr,"ENOENT - segment doesn't exist\n"); break;
		case EACCES: fprintf(stderr,"EACCES - no rights to access segment\n"); break;
		case ENOMEM: fprintf(stderr,"ENOMEM - couldn't reserve memory for segment\n"); break;
	    }
	    return EXIT_FAILURE;
	}

	shm_ptr = shmat(shmid,0,SHM_RDONLY);


	if(shm_ptr==NULL)
	{
	    fprintf(stderr,"ERROR: Failed to shmat\n");
	    return EXIT_FAILURE;
	}


	monitor[cam]->shared_data = shm_ptr;

	monitor[cam]->shared_images = shm_ptr +
	sizeof(SharedData) +
	sizeof(TriggerData) +
	((*monitor[cam]->image_buffer_count)*sizeof(struct timeval));


    }

    unsigned char *data;
    unsigned int pos_data;
    unsigned int pos_rgba;
    unsigned int r,g,b;


    //int cam_in_row=(XDisplayWidth(dis,screen_number)-imgx)/(max_width+5);
    
    int box_num=pow(cam_in_row,2);

    BOX box[16];
    for(cam=0;cam<16;cam++) box[cam].mon_num=-1;
    for(cam=0;cam<cam_num;cam++) box[cam].mon_num=cam;
    
    
    XSelectInput(dis, win, StructureNotifyMask|KeyPressMask);

    XWindowAttributes win_attrib;

    XGetWindowAttributes(dis, win, &win_attrib);

    int win_width=win_attrib.width;
    int win_height=win_attrib.height;
        
    int cam_width=(win_width-imgx)/cam_in_row;
    int cam_height=(win_height-imgy)/cam_in_row;
    int mon=0;
    
    if(xv==0)
    {

        free(rgba);

	rgba=malloc(cam_width*cam_height*4);
        ximage = XCreateImage(dis,XDefaultVisual(dis,screen_number),24,ZPixmap,0,rgba,cam_width,cam_height,32,4*(cam_width));
	if(ximage==NULL)
	{
	    fprintf(stderr,"ERROR: Unable to create ximage\n");
	    return EXIT_FAILURE;
	}
	
    }

    cam=0;
    
    while(1) // loop forever
    {
	if(box[cam].mon_num!=-1)
	{

	    while (monitor[mon]->shared_data->last_write_index==box[cam].lastread)
	    {

		usleep(delay);
		
		if(++cam>=box_num) cam=0;
		while(box[cam].mon_num==-1)
		{
		if(++cam>=box_num) cam=0;
		}
		mon=box[cam].mon_num;
		
		
	    }
	
	    
	    box[cam].lastread=monitor[mon]->shared_data->last_write_index;

	    if(monitor[mon]->pallete==GRAY)
	    data = monitor[mon]->shared_images + (*monitor[mon]->width)*(*monitor[mon]->height)*box[cam].lastread;
	    if(monitor[mon]->pallete==RGB)
	    data = monitor[mon]->shared_images + (*monitor[mon]->width)*(*monitor[mon]->height)*3*box[cam].lastread;
	    //convert
	    pos_rgba=0;

	    if(xv)
	    {	
		if(monitor[mon]->pallete=GRAY)
		{
		    for(pos_data=0;pos_data<((*monitor[mon]->width)*(*monitor[mon]->height)*1);)
		    {
			rgba[pos_rgba++] = data[pos_data]; //b
			rgba[pos_rgba++] = data[pos_data]; //g
			rgba[pos_rgba++] = data[pos_data++]; //r
			pos_rgba++; //a
		    }
		}
		
		if(monitor[mon]->pallete=RGB)
		{
		    for(pos_data=0;pos_data<((*monitor[mon]->width)*(*monitor[mon]->height)*3);)
		    {
			r=(data[pos_data++]);
			g=(data[pos_data++]);
			b=(data[pos_data++]);
		        rgba[pos_rgba++]= b;
			rgba[pos_rgba++] = g;
			rgba[pos_rgba++] = r;
			pos_rgba++;
		    }
		}
		
		
	        XvPutImage(dis,portNum,win,gc,monitor[mon]->image,0,0,*monitor[mon]->width,*monitor[mon]->height,imgx+(cam_width)*(cam%cam_in_row),imgy+(cam_height)*(cam/cam_in_row),cam_width,cam_height);
	    }
	    else
	    {	
	    //software rescalling

		for(y=0;y<cam_height;y++)
		{
		    for(x=0;x<cam_width;x++)
		    {

			pos_data=y*(*monitor[mon]->height)/cam_height;
			pos_data=pos_data*(*monitor[mon]->width);
			pos_data=pos_data+x*(*monitor[mon]->width)/cam_width;

			
			if(monitor[mon]->pallete==GRAY)
			{
			    //pos_data=pos_data*1;
			    rgba[pos_rgba++]= data[pos_data];
			    rgba[pos_rgba++] = data[pos_data];
			    rgba[pos_rgba++] = data[pos_data++];
			    rgba[pos_rgba++] = 0;
			}
			if(monitor[mon]->pallete==RGB)
			{

			    pos_data=pos_data*3;
			    r=(data[pos_data++]);
			    g=(data[pos_data++]);
			    b=(data[pos_data++]);
			    rgba[pos_rgba++]=b;
			    rgba[pos_rgba++]=g;
			    rgba[pos_rgba++]=r;
			    rgba[pos_rgba++]=0;
			}
		    }		
		
		}		

		XPutImage(dis,win,gc,ximage,0,0,imgx+(cam_width)*(cam%cam_in_row),imgy+(cam_height)*(cam/cam_in_row),cam_width,cam_height);	    
	    }
	}
	
        while(XPending(dis))
	{
	    XNextEvent(dis, &event);
	    if(event.type == ConfigureNotify)
	    {
		win_width = event.xconfigure.width;
		win_height = event.xconfigure.height;
		cam_width=(win_width-imgx)/cam_in_row;
		cam_height=(win_height-imgy)/cam_in_row;
		XClearWindow (dis, win);
		if(xv==0)
		{
		    XDestroyImage(ximage);
		    rgba=malloc(cam_width*cam_height*4);
		    ximage = XCreateImage(dis,XDefaultVisual(dis,screen_number),24,ZPixmap,0,rgba,cam_width,cam_height,32,4*(cam_width));
		}

	    }
	    if(event.type == ClientMessage)
	    {
		if(event.xclient.data.l[0]==wm_delete_window)
		{
		    fprintf(stdout,"Shutdown\n");
		    exit(EXIT_SUCCESS);
		}
	    }
	    if(event.type == KeyPress)
	    {
		fprintf(stdout,"Key pressed:%d\n",event.xkey.keycode);
		
		int_temp=0;
		switch(event.xkey.keycode)
		{
		    case key1:
			int_temp=1;
			break;
		    case key2:
			int_temp=2;
			break;
		    case key3:
			int_temp=3;
			break;
		    case key4:
			int_temp=4;
			break;

		    case keyq:
			int_temp=5;
			break;
		    case keyw:
			int_temp=6;
			break;
		    case keye:
			int_temp=7;
			break;
		    case keyr:
			int_temp=8;
			break;

		    case keya:
			int_temp=9;
			break;
		    case keys:
			int_temp=10;
			break;
		    case keyd:
			int_temp=11;
			break;
		    case keyf:
			int_temp=12;
			break;

		    case keyz:
			int_temp=13;
			break;
		    case keyx:
			int_temp=14;
			break;
		    case keyc:
			int_temp=15;
			break;
		    case keyv:
			int_temp=16;
			break;
		    
		    //mode
		    case key5:
			int_temp=17;
			break;
		    case keyt:
			int_temp=18;
			break;
		    case keyg:
			int_temp=19;
			break;
		    case keyb:
			int_temp=20;
			break;
		}
		
		if(int_temp>0&&int_temp<17)
		{
		    
		    int_temp=int_temp-(int_temp/4)*(4-cam_in_row);
		    if(++box[int_temp-1].mon_num>cam_num-1) box[int_temp-1].mon_num=-1;
    		    box[int_temp-1].lastread=0;
		    //first box must be enabled!
		    if(box[0].mon_num==-1) box[0].mon_num=0;
		    cam=0;
		    XClearWindow (dis, win);
		}
		
		if(int_temp>16&&int_temp<21)
		{
		    cam_in_row=int_temp-16;
		    cam_width=(win_width-imgx)/cam_in_row;
		    cam_height=(win_height-imgy)/cam_in_row;
		    box_num=pow(cam_in_row,2);
		    cam=0;
		    XClearWindow (dis, win);
		    if(xv==0)
		    {
			XDestroyImage(ximage);
			rgba=malloc(cam_width*cam_height*4);
			ximage = XCreateImage(dis,XDefaultVisual(dis,screen_number),24,ZPixmap,0,rgba,cam_width,cam_height,32,4*(cam_width));
		    }
		    
		}
	    }
	}
    }
}
